@@ -1,15 +1,16 @@
 """
 Custom form fields for URLs
 """
-import copy
+import django
 from django import forms
 from django.core import validators
 from django.core.exceptions import ValidationError
-from django.db.models.base import Model, django
-from django.forms.util import ErrorList
-from django.utils.translation import ugettext_lazy as _
+from django.db.models.base import Model
+from django.forms.utils import ErrorList
+from django.utils.translation import gettext_lazy as _
+
 from any_urlfield.forms.widgets import AnyUrlWidget
-from any_urlfield.models.values import AnyUrlValue
+from any_urlfield.validators import ExtendedURLValidator
 
 
 class AnyUrlField(forms.MultiValueField):
@@ -21,7 +22,6 @@ class AnyUrlField(forms.MultiValueField):
     """
     widget = AnyUrlWidget
 
-
     def __init__(self, url_type_registry, max_length=None, *args, **kwargs):
         self.url_type_registry = url_type_registry  # UrlTypeRegistry object
 
@@ -31,22 +31,26 @@ class AnyUrlField(forms.MultiValueField):
         choices = []
         for urltype in self.url_type_registry:
             # Get formfield, update properties
-            field = urltype.form_field
+            field = urltype.get_form_field()
             field.required = False   # Delay check, happens somewhere else.
             if getattr(field, 'max_length', None) and field.max_length > max_length:
                 field.max_length = max_length
 
             fields.append(field)
             choices.append((urltype.prefix, urltype.title))
-        fields.insert(0, forms.ChoiceField(label=_("Type URL"), choices=choices))
+        fields.insert(0, forms.ChoiceField(label=_("Type URL"), choices=choices, initial=choices[0][0]))
 
         # Instantiate widget. Is not done by parent at all.
         widget = self.widget(url_type_registry=url_type_registry)
         kwargs['widget'] = widget
-        super(AnyUrlField, self).__init__(fields, *args, **kwargs)
-
+        kwargs.pop('empty_value', None)  # for Django 1.11
+        super().__init__(fields, *args, **kwargs)
 
     def compress(self, data_list):
+        # Reimporting models from froms is tricky, and may lead to circular ImportErrors
+        # Hence, importing here locally.
+        from any_urlfield.models.values import AnyUrlValue
+
         if data_list:
             type_prefix = data_list[0]    # avoid `id, *values = data_list` notation, that is python 3 syntax.
             values = data_list[1:]
@@ -66,13 +70,12 @@ class AnyUrlField(forms.MultiValueField):
                     if isinstance(value, Model):
                         value = value.pk   # Auto cast foreign keys to integer.
                     elif value:
-                        value = long(value)
+                        value = int(value)
                     else:
                         return None
                 return AnyUrlValue(type_prefix, value, self.url_type_registry)
         return None
 
-
     def clean(self, value):
         # Get the value
         # Totally replaced validation.
@@ -105,7 +108,7 @@ class AnyUrlField(forms.MultiValueField):
 
                 try:
                     clean_data.append(field.clean(field_value))
-                except ValidationError, e:
+                except ValidationError as e:
                     errors.extend(e.messages)  # Collect all widget errors
             if errors:
                 raise ValidationError(errors)
@@ -115,11 +118,17 @@ class AnyUrlField(forms.MultiValueField):
         self.validate(out)
         return out
 
-    if django.VERSION < (1,7):
-        # ModelChoiceField.__deepcopy__ was skipped because this object didn't deepcopy the fields.
-        # That causes issues when the queryset needs to be reevaluated (e.g. for queries with a SITE_ID threadlocal).
+    def has_changed(self, initial, data):
+        # special case, except when no data was set, because our decompress()
+        # will cause a comparison between ['http', '', ''] and ['', '', ''].
+        if initial is None and not any(data[1:]):
+            return False
 
-        def __deepcopy__(self, memo):
-            result = super(AnyUrlField, self).__deepcopy__(memo)
-            result.fields = copy.deepcopy(result.fields, memo)
-            return result
+        return super().has_changed(initial, data)
+
+
+class ExtendedURLField(forms.URLField):
+    """
+    An URL field that also supports validating ``tel:`` and ``mailto:`` links.
+    """
+    default_validators = [ExtendedURLValidator]
