@@ -1,14 +1,15 @@
-from types import LambdaType
 from django import forms
-from django.utils.translation import ugettext_lazy as _
+from django.core.cache import cache
+from django.db.models import signals
+from django.utils.translation import gettext_lazy as _
+
+from any_urlfield import EXTERNAL_SCHEMES
+from any_urlfield.cache import get_object_cache_keys
+from any_urlfield.forms.fields import ExtendedURLField
 
 
-# Avoid using common protocol names as prefix, this could clash in the future.
-# Values starting with such prefix should be handled as external URL.
-_invalid_prefixes = ('http', 'https', 'ftp', 'ftps', 'sftp', 'webdav', 'webdavs', 'afp', 'smb', 'git', 'svn', 'hg')
+class UrlType:
 
-
-class UrlType(object):
     def __init__(self, model, form_field, widget, title, prefix, has_id_value):
         if form_field is None:
             # Generate default form field if nothing is provided.
@@ -24,7 +25,7 @@ class UrlType(object):
         self.has_id_value = has_id_value
 
     def __repr__(self):
-        return "<UrlType {0}>".format(self.prefix)
+        return "<UrlType {}>".format(self.prefix)
 
     def __getstate__(self):
         # Can't pickle lambda or callable values, so force evaluation
@@ -33,17 +34,16 @@ class UrlType(object):
         return dict
 
     def __eq__(self, other):
+        if not isinstance(other, UrlType):
+            return NotImplemented
+
         # Skipping title and form_field
-        return isinstance(other, UrlType) \
-           and self.model == other.model \
-           and self.prefix == other.prefix \
-           and self.has_id_value == other.has_id_value
+        return self.model == other.model \
+            and self.prefix == other.prefix \
+            and self.has_id_value == other.has_id_value
 
     def __ne__(self, other):
-        return not isinstance(other, UrlType) \
-            or self.model != other.model \
-            or self.prefix != other.prefix \
-            or self.has_id_value != other.has_id_value
+        return not self == other
 
     def get_form_field(self):
         """
@@ -72,8 +72,7 @@ class UrlType(object):
         return widget
 
 
-
-class UrlTypeRegistry(object):
+class UrlTypeRegistry:
     """
     Registration backend to administrate the various types.
     """
@@ -81,63 +80,67 @@ class UrlTypeRegistry(object):
     def __init__(self):
         self._url_types = [UrlType(
             model=None,
-            form_field=forms.URLField(label=_("External URL"), widget=forms.TextInput(attrs={'class': 'vTextField'})),
+            form_field=ExtendedURLField(label=_("External URL"), widget=forms.TextInput(attrs={'class': 'vTextField'})),
             widget=None,
             title=_("External URL"),
             prefix='http',   # no https needed, 'http' is a special constant.
             has_id_value=False
         )]
 
-
     def register(self, ModelClass, form_field=None, widget=None, title=None, prefix=None, has_id_value=True):
         """
         Register a custom model with the ``AnyUrlField``.
         """
         if any(urltype.model == ModelClass for urltype in self._url_types):
-            raise ValueError("Model is already registered: '{0}'".format(ModelClass))
+            raise ValueError("Model is already registered: '{}'".format(ModelClass))
 
         opts = ModelClass._meta
-        try:
-            opts = opts.concrete_model._meta
-        except AttributeError:  # Django 1.3
-            pass
+        opts = opts.concrete_model._meta
 
         if not prefix:
             # Store something descriptive, easier to lookup from raw database content.
-            prefix = '{0}.{1}'.format(opts.app_label, opts.object_name.lower())
+            prefix = '{}.{}'.format(opts.app_label, opts.object_name.lower())
         if not title:
             title = ModelClass._meta.verbose_name
 
         if self.is_external_url_prefix(prefix):
-            raise ValueError("Invalid prefix value: '{0}'.".format(prefix))
+            raise ValueError("Invalid prefix value: '{}'.".format(prefix))
         if self[prefix] is not None:
-            raise ValueError("Prefix is already registered: '{0}'".format(prefix))
+            raise ValueError("Prefix is already registered: '{}'".format(prefix))
         if form_field is not None and widget is not None:
             raise ValueError("Provide either a form_field or widget; use the widget parameter of the form field instead.")
 
         urltype = UrlType(ModelClass, form_field, widget, title, prefix, has_id_value)
+        signals.post_save.connect(_on_model_save, sender=ModelClass)
         self._url_types.append(urltype)
         return urltype
 
-
     def is_external_url_prefix(self, prefix):
-        return prefix in _invalid_prefixes
-
+        return prefix in EXTERNAL_SCHEMES
 
     def __eq__(self, other):
+        if not isinstance(other, UrlTypeRegistry):
+            return NotImplemented
+
         # For __getstate__ logic
         return self._url_types == other._url_types
 
-
     def __ne__(self, other):
-        return self._url_types != other._url_types
-
+        return not self == other
 
     # Accessing API is similar to `list` and '`dict`:
 
     def __iter__(self):
         return iter(self._url_types)
 
+    def get_for_model(self, ModelClass):
+        """
+        Return the URL type for a given model class
+        """
+        for urltype in self._url_types:
+            if urltype.model is ModelClass:
+                return urltype
+        return None
 
     def __getitem__(self, prefix):
         # Any web domain will be handled by the standard URLField.
@@ -149,7 +152,6 @@ class UrlTypeRegistry(object):
                 return urltype
         return None
 
-
     def index(self, prefix):
         """
         Return the model index for a prefix.
@@ -163,9 +165,15 @@ class UrlTypeRegistry(object):
                 return i
         return None
 
-
     def keys(self):
         """
         Return the available url type prefixes.
         """
         return [urltype.prefix for urltype in self._url_types]
+
+
+def _on_model_save(instance, **kwargs):
+    """
+    Called when a model is saved.
+    """
+    cache.delete_many(get_object_cache_keys(instance))
